package edu.nyu.scps.plumb;

import android.content.Context;
import android.content.res.Resources;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.PointF;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.util.DisplayMetrics;
import android.util.TypedValue;
import android.view.Display;
import android.view.Surface;
import android.view.View;
import android.view.WindowManager;


public class PlumbView extends View implements SensorEventListener {
    private Paint paint = new Paint(Paint.ANTI_ALIAS_FLAG);
    private PointF acceleration = new PointF(0f, 1f);
    private Display defaultDisplay;

    public PlumbView(Context context) {
        super(context);
        paint.setColor(Color.BLACK);
        paint.setStyle(Paint.Style.FILL);

        //Convert 24sp to pixels.
        Resources resources = getResources();
        DisplayMetrics displayMetrics = resources.getDisplayMetrics();
        float size = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_SP, 24f, displayMetrics);
        paint.setTextSize(size);

        WindowManager windowManager = (WindowManager)context.getSystemService(Context.WINDOW_SERVICE);
        defaultDisplay = windowManager.getDefaultDisplay();
    }

    @Override
    public void onSensorChanged(SensorEvent sensorEvent) {
        acceleration.set(sensorEvent.values[0], sensorEvent.values[1]);
        invalidate();   //Trigger a call on onDraw.
    }

    @Override
    public void onAccuracyChanged(Sensor sensor, int accuracy) {
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);

        double theta;
        String rotation;
        switch (defaultDisplay.getRotation()) {
            case Surface.ROTATION_0:
                rotation = "ROTATION_0";
                theta = Math.atan2(-acceleration.y, -acceleration.x);
                break;
            case Surface.ROTATION_90:
                rotation = "ROTATION_90";
                theta = Math.atan2(-acceleration.x, acceleration.y);
                break;
            case Surface.ROTATION_180:
                rotation = "ROTATION_180";
                theta = Math.atan2(acceleration.y, acceleration.x);
                break;
            case Surface.ROTATION_270:
                rotation = "ROTATION_270";
                theta = Math.atan2(acceleration.x, -acceleration.y);
                break;
            default:
                rotation = "ROTATION_?";
                theta = 0;
                break;
        }

        int width = getWidth();                         //of PlumbView
        int height = getHeight();                       //of PlumbView
        float length = .375f * Math.min(width, height); //of string
        float radius = .1f * length;                    //of circle
        float x = length * (float)Math.cos(theta);
        float y = length * (float)Math.sin(theta);

        //At this point, origin is in upper left corner of PlumbView, Y axis points down.
        //03B8 is lowercase Greek theta, 00B0 is degree symbol.
        float size = paint.getTextSize();
        float indent = size / 4f;
        canvas.drawText(rotation, indent, size, paint);
        canvas.drawText("X acceleration = " + acceleration.x, indent, 2f * size, paint);
        canvas.drawText("Y acceleration = " + acceleration.y, indent, 3f * size, paint);
        canvas.drawText("\u03B8 = " + Math.round(Math.toDegrees(theta)) + "\u00B0",
                indent, 4f * size, paint);

        //Origin in center of PlumbView, Y axis points up.
        canvas.translate(width / 2f, height / 2f);
        canvas.scale(1f, -1f);

        //The plumb bob hangs down from the center of the PlumbView.
        canvas.drawLine(0f, 0f, x, y, paint);
        canvas.drawCircle(x, y, radius, paint);
    }
}